@@ -2,7 +2,7 @@
    |                     Mobile Robot Programming Toolkit (MRPT)            |
    |                          https://www.mrpt.org/                         |
    |                                                                        |
-   | Copyright (c) 2005-2020, Individual contributors, see AUTHORS file     |
+   | Copyright (c) 2005-2023, Individual contributors, see AUTHORS file     |
    | See: https://www.mrpt.org/Authors - All rights reserved.               |
    | Released under BSD License. See: https://www.mrpt.org/License          |
    +------------------------------------------------------------------------+ */
@@ -15,10 +15,12 @@
 #include <mrpt/core/demangle.h>
 #include <mrpt/core/exceptions.h>
 #include <mrpt/core/format.h>
+#include <mrpt/typemeta/TEnumType.h>
 
 #include <any>
 #include <array>
 #include <cstdint>
+#include <cstdlib>
 #include <iosfwd>
 #include <limits>
 #include <map>
@@ -74,6 +76,9 @@ namespace mrpt::containers
  * Output:
  *  \include containers_yaml_example/console.out
  *
+ * Verbose debug information on YAML document parsing is emitted if the
+ * environment variable `MRPT_YAML_PARSER_VERBOSE` is set to `1`.
+ *
  * \ingroup mrpt_containers_yaml
  * \note [New in MRPT 2.1.0]
  */
@@ -86,26 +91,49 @@ class yaml
 	struct node_t;
 	using scalar_t = std::any;
 	using sequence_t = std::vector<node_t>;
-	using map_t = std::map<node_t, node_t, std::less<> /*transparent comp*/>;
+	using map_t = std::map<node_t, node_t>;
 
 	using comments_t = std::array<
 		std::optional<std::string>, static_cast<size_t>(CommentPosition::MAX)>;
 
+	struct mark_t
+	{
+		/// Position from the start of the input file
+		std::size_t input_pos = 0;
+		int line = 0;  //!< Line position (0-based index)
+		int column = 0;	 //!< Column  position (0-based index)
+	};
+
 	struct node_t
 	{
+		/** @name Data
+		 *  @{ */
+
 		/** Node data */
 		std::variant<std::monostate, sequence_t, map_t, scalar_t> d;
 
 		/** Optional comment block */
 		comments_t comments;
 
+		/** Optional flag to print collections in short form (e.g. [A,B] for
+		 * sequences) \note (New in MRPT 2.1.8) */
+		bool printInShortFormat = false;
+
+		/** Positioning information about the placement of the element in the
+		 * original input file/stream, i.e. line and column number
+		 * \note (New in MRPT 2.5.0)
+		 */
+		mark_t marks;
+
+		/** @} */
+
 		node_t() = default;
 		~node_t() = default;
 
 		template <
-			typename T,  //
+			typename T,	 //
 			typename = std::enable_if_t<!std::is_constructible_v<
-				std::initializer_list<map_t::value_type>, T>>,  //
+				std::initializer_list<map_t::value_type>, T>>,	//
 			typename = std::enable_if_t<!std::is_constructible_v<
 				std::initializer_list<sequence_t::value_type>, T>>>
 		node_t(const T& scalar)
@@ -155,11 +183,11 @@ class yaml
 		 * sequence or map nodes. */
 		size_t size() const;
 
-		/** Returns a copy of the existing value of the given type, or tries to
-		 * convert it between easily-compatible types (e.g. double<->int,
+		/** Returns a copy of the existing value of the given type, or tries
+		 * to convert it between easily-compatible types (e.g. double<->int,
 		 * string<->int).
-		 * \exception std::exception If the contained type does not  match and
-		 * there is no obvious conversion.
+		 * \exception std::exception If the contained type does not  match
+		 * and there is no obvious conversion.
 		 */
 		template <typename T>
 		T as() const
@@ -193,10 +221,47 @@ class yaml
 				return {*s};
 			}
 			THROW_EXCEPTION_FMT(
-				"Used node_t as map key with a type non-convertible to string: "
+				"Used node_t as map key with a type non-convertible to "
+				"string: "
 				"'%s'",
 				typeName().c_str());
 		}
+
+		bool hasComment() const
+		{
+			for (const auto& c : comments)
+				if (c.has_value()) return true;
+			return false;
+		}
+		bool hasComment(CommentPosition pos) const
+		{
+			MRPT_START
+			int posIndex = static_cast<int>(pos);
+			ASSERT_GE_(posIndex, 0);
+			ASSERT_LT_(posIndex, static_cast<int>(CommentPosition::MAX));
+			return comments[posIndex].has_value();
+			MRPT_END
+		}
+		const std::string& comment() const
+		{
+			MRPT_START
+			for (const auto& c : comments)
+				if (c.has_value()) return c.value();
+			THROW_EXCEPTION("Trying to access comment but this node has none.");
+			MRPT_END
+		}
+		const std::string& comment(CommentPosition pos) const
+		{
+			MRPT_START
+			int posIndex = static_cast<int>(pos);
+			ASSERT_GE_(posIndex, 0);
+			ASSERT_LT_(posIndex, static_cast<int>(CommentPosition::MAX));
+			ASSERTMSG_(
+				comments[posIndex].has_value(),
+				"Trying to access comment but this node has none.");
+			return comments[posIndex].value();
+			MRPT_END
+		}
 	};
 
 	/** @} */
@@ -252,8 +317,8 @@ class yaml
 	 */
 	void loadFromText(const std::string& yamlTextBlock);
 
-	/** Parses the stream as YAML or JSON (autodetected) and returns a document.
-	 * \exception std::exception Upon format errors
+	/** Parses the stream as YAML or JSON (autodetected) and returns a
+	 * document. \exception std::exception Upon format errors
 	 */
 	static yaml FromStream(std::istream& i);
 
@@ -270,16 +335,16 @@ class yaml
 	 */
 	static yaml FromFile(const std::string& fileName);
 
-	/** Parses the stream as YAML or JSON (autodetected) and stores the contents
-	 * into this document.
+	/** Parses the stream as YAML or JSON (autodetected) and stores the
+	 * contents into this document.
 	 *
 	 * \exception std::exception Upon format errors
 	 */
 	void loadFromStream(std::istream& i);
 
 	/** Builds an object copying the structure and contents from an existing
-	 * YAMLCPP Node. Requires user to #include yamlcpp from your calling program
-	 * (does NOT requires yamlcpp while compiling mrpt itself).
+	 * YAMLCPP Node. Requires user to #include yamlcpp from your calling
+	 * program (does NOT requires yamlcpp while compiling mrpt itself).
 	 *
 	 * \tparam YAML_NODE Must be `YAML::Node`. Made a template just to avoid
 	 * build-time depedencies.
@@ -291,19 +356,53 @@ class yaml
 	template <typename YAML_NODE>
 	inline void loadFromYAMLCPP(const YAML_NODE& n);
 
+	/** Creates a yaml dictionary node from an Eigen or mrpt::math matrix.
+	 * Example (compatible with OpenCV & ROS YAML formats):
+	 * \code
+	 * rows: 2
+	 * cols: 3
+	 * data: [11, 12, 13, 21, 22, 23]
+	 * \endcode
+	 * \sa toMatrix()
+	 */
+	template <typename MATRIX>
+	inline static yaml FromMatrix(const MATRIX& m);
+
+	/** Fills in a matrix from a yaml dictionary node.
+	 * The matrix can be either an Eigen or mrpt::math matrix.
+	 * Example yaml node (compatible with OpenCV & ROS YAML formats):
+	 * \code
+	 * rows: 2
+	 * cols: 3
+	 * data: [11, 12, 13, 21, 22, 23]
+	 * \endcode
+	 * \sa FromMatrix()
+	 */
+	template <typename MATRIX>
+	inline void toMatrix(MATRIX& m) const;
+
+	/** Converts a sequence yaml node into a std::vector, trying to convert
+	 * all nodes to the same given `Scalar` type. \note (New in MRPT 2.3.3)
+	 */
+	template <typename Scalar>
+	inline std::vector<Scalar> toStdVector() const;
+
 	/** @} */
 
 	/** @name Content and type checkers
 	 * @{ */
-	/** For map nodes, checks if the given key name exists */
+	/** For map nodes, checks if the given key name exists.
+	 *  Returns false if the node is a `null` node.
+	 *  Throws if the node is not a map or null.
+	 */
 	bool has(const std::string& key) const;
 
 	/** For map or sequence nodes, checks if the container is empty. Also
 	 * returns true for null(empty) nodes. */
 	bool empty() const;
 
-	/** Resets to empty (can be called on a root node or any other node to clear
-	 * that subtree only). */
+	/** Resets to empty (can be called on a root node or any other node to
+	 * clear that subtree only). */
 	void clear();
 
 	bool isNullNode() const;
@@ -311,8 +410,8 @@ class yaml
 	bool isSequence() const;
 	bool isMap() const;
 
-	/** For scalar nodes, returns its type, or typeid(void) if an empty node.
-	 * \exception std::exception If called on a map or sequence. */
+	/** For scalar nodes, returns its type, or typeid(void) if an empty
+	 * node. \exception std::exception If called on a map or sequence. */
 	const std::type_info& scalarType() const;
 
 	/** @} */
@@ -334,8 +433,8 @@ class yaml
 	scalar_t& asScalar();
 	const scalar_t& asScalar() const;
 
-	/** Returns 1 for null or scalar nodes, the number of children for sequence
-	 * or map nodes. */
+	/** Returns 1 for null or scalar nodes, the number of children for
+	 * sequence or map nodes. */
 	size_t size() const;
 
 	/** For a master yaml document, returns the root node; otherwise, the
@@ -401,7 +500,8 @@ class yaml
 		if (n->isNullNode()) return defaultValue;
 		if (!n->isMap())
 			THROW_EXCEPTION_FMT(
-				"getOrDefault() is only for map nodes, invoked on a node of "
+				"getOrDefault() is only for map nodes, invoked on a node "
+				"of "
 				"type: '%s'",
 				n->typeName().c_str());
 
@@ -416,7 +516,8 @@ class yaml
 		catch (const std::bad_any_cast& e)
 		{
 			throw std::logic_error(mrpt::format(
-				"getOrDefault(): Trying to access key `%s` holding type `%s` "
+				"getOrDefault(): Trying to access key `%s` holding type "
+				"`%s` "
 				"as the wrong type: `%s`",
 				key.c_str(), n->typeName().c_str(), e.what()));
 		}
@@ -433,13 +534,13 @@ class yaml
 
 	/** Append a new value to a sequence.
 	 * \throw std::exception If this is not a sequence */
-	void push_back(const double v) { internalPushBack(v); }
+	void push_back(double v) { internalPushBack(v); }
 	/// \overload
-	void push_back(const std::string v) { internalPushBack(v); }
+	void push_back(const std::string& v) { internalPushBack(v); }
 	/// \overload
-	void push_back(const uint64_t v) { internalPushBack(v); }
+	void push_back(uint64_t v) { internalPushBack(v); }
 	/// \overload
-	void push_back(const bool v) { internalPushBack(bool(v)); }
+	void push_back(bool v) { internalPushBack(bool(v)); }
 	/// \overload
 	void push_back(const yaml& v)
 	{
@@ -459,8 +560,8 @@ class yaml
 	/** @name Internal proxy
 	 * @{ */
 
-	/** Returns the pointer to the referenced node data, if a proxy, or to the
-	 * root node otherwise. Will never return nullptr. */
+	/** Returns the pointer to the referenced node data, if a proxy, or to
+	 * the root node otherwise. Will never return nullptr. */
 	const node_t* dereferenceProxy() const;
 	node_t* dereferenceProxy();
 
@@ -490,8 +591,8 @@ class yaml
 	/** Returns a copy of the existing value of the given type, or tries to
 	 * convert it between easily-compatible types (e.g. double<->int,
 	 * string<->int).
-	 * \exception std::exception If the contained type does not  match and there
-	 * is no obvious conversion.
+	 * \exception std::exception If the contained type does not  match and
+	 * there is no obvious conversion.
 	 */
 	template <typename T>
 	T as() const
@@ -499,17 +600,17 @@ class yaml
 		return internal::implAsGetter<T>(*this);
 	}
 
-	/** Returns a ref to the existing or new value of the given type. If types
-	 * do not match, the old content will be discarded and a new variable
-	 * created into this scalar node.
-	 * \exception std::exception If accessing to a non-scalar node.
+	/** Returns a ref to the existing or new value of the given type. If
+	 * types do not match, the old content will be discarded and a new
+	 * variable created into this scalar node. \exception std::exception If
+	 * accessing to a non-scalar node.
 	 */
 	template <typename T>
 	T& asRef();
 
-	/** const version of asRef(). Unlike `as<T>()`, this version will NOT try to
-	 * convert between types if T does not match exactly the stored type, and
-	 * will raise an exception instead. */
+	/** const version of asRef(). Unlike `as<T>()`, this version will NOT
+	 * try to convert between types if T does not match exactly the stored
+	 * type, and will raise an exception instead. */
 	template <typename T>
 	const T& asRef() const;
 
@@ -527,6 +628,22 @@ class yaml
 	yaml& operator=(int64_t v);
 	yaml& operator=(uint64_t v);
 
+	// Additional operator for "size_t", in systems/compilers where
+	// size_t != all other types above
+	// (e.g. OSX with clang, see
+	// https://stackoverflow.com/a/11603907/1631514 )
+	template <
+		typename = std::enable_if<
+			!std::is_same_v<std::size_t, uint64_t> &&
+			!std::is_same_v<std::size_t, int64_t> &&
+			!std::is_same_v<std::size_t, uint32_t> &&
+			!std::is_same_v<std::size_t, int32_t>>	//
+		>
+	yaml& operator=(std::size_t v)
+	{
+		return operator=(static_cast<uint64_t>(v));
+	}
+
 	yaml& operator=(const std::string& v);
 	inline yaml& operator=(const char* v) { return operator=(std::string(v)); }
 	inline yaml& operator=(const std::string_view& v)
@@ -552,7 +669,8 @@ class yaml
 		if (isNullNode()) node().d.emplace<map_t>();
 		ASSERTMSG_(
 			isMap(),
-			"<< operator with ValueKeyCommentPair requires a map (dictionary) "
+			"<< operator with ValueKeyCommentPair requires a map "
+			"(dictionary) "
 			"on the left hand.");
 		operator[](vc.keyname) = vc.value;
 		// keyComment:
@@ -584,16 +702,16 @@ class yaml
 	/** @name Leaf node comments API
 	 * @{ */
 
-	/** Returns true if the proxied node has an associated comment block, at any
-	 * location */
+	/** Returns true if the proxied node has an associated comment block, at
+	 * any location */
 	bool hasComment() const;
 
-	/** Returns true if the proxied node has an associated comment block at a
-	 * particular position */
+	/** Returns true if the proxied node has an associated comment block at
+	 * a particular position */
 	bool hasComment(CommentPosition pos) const;
 
-	/** Gets the comment associated to the proxied node. This version returns
-	 * the first comment, of all possible (top, right).
+	/** Gets the comment associated to the proxied node. This version
+	 * returns the first comment, of all possible (top, right).
 	 *
 	 * \exception std::exception If there is no comment attached.
 	 * \sa hasComment()
@@ -621,33 +739,33 @@ class yaml
 	/** @name Map key node comments API
 	 * @{ */
 
-	/** Maps only: returns true if the given key node has an associated comment
-	 * block, at any location.
-	 * \exception std::exception If called on a non-map or key does not exist.
+	/** Maps only: returns true if the given key node has an associated
+	 * comment block, at any location. \exception std::exception If called
+	 * on a non-map or key does not exist.
 	 */
 	bool keyHasComment(const std::string& key) const;
 
-	/** Maps only: Returns true if the given key has an associated comment block
-	 * at a particular position.
-	 * \exception std::exception If called on a non-map or key does not exist.
+	/** Maps only: Returns true if the given key has an associated comment
+	 * block at a particular position. \exception std::exception If called
+	 * on a non-map or key does not exist.
 	 */
 	bool keyHasComment(const std::string& key, CommentPosition pos) const;
 
-	/** Maps only: Gets the comment associated to the given key. This version
-	 * returns the first comment, of all possible (top, right).
+	/** Maps only: Gets the comment associated to the given key. This
+	 * version returns the first comment, of all possible (top, right).
 	 *
-	 * \exception std::exception If called on a non-map or key does not exist.
-	 * \exception std::exception If there is no comment attached.
-	 * \sa hasComment()
+	 * \exception std::exception If called on a non-map or key does not
+	 * exist. \exception std::exception If there is no comment attached. \sa
+	 * hasComment()
 	 */
 	const std::string& keyComment(const std::string& key) const;
 
 	/** Maps only: Gets the comment associated to the given key, at the
 	 * particular position. See code examples in mrpt::containers::yaml.
 	 *
-	 * \exception std::exception If called on a non-map or key does not exist.
-	 * \exception std::exception If there is no comment attached.
-	 * \sa hasComment()
+	 * \exception std::exception If called on a non-map or key does not
+	 * exist. \exception std::exception If there is no comment attached. \sa
+	 * hasComment()
 	 */
 	const std::string& keyComment(
 		const std::string& key, CommentPosition pos) const;
@@ -655,8 +773,8 @@ class yaml
 	/** Maps only: Sets the comment attached to a given key.
 	 * See code examples in mrpt::containers::yaml
 	 *
-	 * \exception std::exception If called on a non-map or key does not exist.
-	 * \sa hasComment()
+	 * \exception std::exception If called on a non-map or key does not
+	 * exist. \sa hasComment()
 	 */
 	void keyComment(
 		const std::string& key, const std::string& c,
@@ -677,6 +795,7 @@ class yaml
 		int indent = 0;
 		bool needsNL = false;
 		bool needsSpace = false;
+		bool shortFormat = false;
 	};
 
 	// Return: true if the last printed char is a newline char
@@ -723,8 +842,8 @@ class yaml
 };
 
 /** Prints a scalar, a part of a yaml tree, or the entire structure,
- * in YAML-like format. This version does NOT emit neither the YAML header nor
- * the final end line.
+ * in YAML-like format. This version does NOT emit neither the YAML header
+ * nor the final end line.
  *
  * \sa yaml::PrintAsYAML
  */
@@ -741,14 +860,20 @@ std::ostream& operator<<(std::ostream& o, const yaml& p);
  *
  * MCP_LOAD_REQ(p, K);
  * \endcode
+ *
+ * Since MRPT 2.3.2, this also works for enums, converting to textual names of
+ * values. Note that this requires enums to implement mrpt::typemeta::TEnumType.
  */
-#define MCP_LOAD_REQ(paramsVariable__, keyproxiedMapEntryName__)           \
-	if (!paramsVariable__.has(#keyproxiedMapEntryName__))                  \
-		throw std::invalid_argument(mrpt::format(                          \
-			"Required parameter `%s` not an existing key in dictionary.",  \
-			#keyproxiedMapEntryName__));                                   \
-	keyproxiedMapEntryName__ = paramsVariable__[#keyproxiedMapEntryName__] \
-								   .as<decltype(keyproxiedMapEntryName__)>()
+#define MCP_LOAD_REQ(Yaml__, Var__)                                            \
+	if (!Yaml__.has(#Var__))                                                   \
+		throw std::invalid_argument(mrpt::format(                              \
+			"Required parameter `%s` not an existing key in dictionary.",      \
+			#Var__));                                                          \
+	if constexpr (std::is_enum_v<decltype(Var__)>)                             \
+		Var__ = mrpt::typemeta::TEnumType<std::remove_cv_t<decltype(Var__)>>:: \
+			name2value(Yaml__[#Var__].as<std::string>());                      \
+	else                                                                       \
+		Var__ = Yaml__[#Var__].as<decltype(Var__)>()
 
 /** Macro to load a variable from a mrpt::containers::yaml (initials MCP)
  * dictionary, leaving it with its former value if not found (OPTional).
@@ -760,23 +885,32 @@ std::ostream& operator<<(std::ostream& o, const yaml& p);
  *
  * MCP_LOAD_OPT(p, K);
  * \endcode
+ *
+ * Since MRPT 2.3.2, this also works for enums, converting to textual names of
+ * values. Note that this requires enums to implement mrpt::typemeta::TEnumType.
  */
-#define MCP_LOAD_OPT(paramsVariable__, keyproxiedMapEntryName__) \
-	keyproxiedMapEntryName__ = paramsVariable__.getOrDefault(    \
-		#keyproxiedMapEntryName__, keyproxiedMapEntryName__)
+#define MCP_LOAD_OPT(Yaml__, Var__)                                            \
+	if constexpr (std::is_enum_v<decltype(Var__)>)                             \
+	{                                                                          \
+		if (!Yaml__.empty() && Yaml__.has(#Var__))                             \
+			Var__ = mrpt::typemeta::TEnumType<std::remove_cv_t<decltype(       \
+				Var__)>>::name2value(Yaml__[#Var__].as<std::string>());        \
+	}                                                                          \
+	else if (!Yaml__.isNullNode() && !Yaml__.empty() && Yaml__.has(#Var__))    \
+	Var__ = Yaml__[#Var__].as<decltype(Var__)>()
 
 /** Just like MCP_LOAD_REQ(), but converts the read number from degrees to
  * radians */
-#define MCP_LOAD_REQ_DEG(paramsVariable__, keyproxiedMapEntryName__) \
-	MCP_LOAD_REQ(paramsVariable__, keyproxiedMapEntryName__);        \
-	keyproxiedMapEntryName__ = mrpt::DEG2RAD(keyproxiedMapEntryName__)
+#define MCP_LOAD_REQ_DEG(Yaml__, Var__)                                        \
+	MCP_LOAD_REQ(Yaml__, Var__);                                               \
+	Var__ = mrpt::DEG2RAD(Var__)
 
 /** Just like MCP_LOAD_OPT(), but converts the read number from degrees to
  * radians */
-#define MCP_LOAD_OPT_DEG(paramsVariable__, keyproxiedMapEntryName__)    \
-	keyproxiedMapEntryName__ = mrpt::RAD2DEG(keyproxiedMapEntryName__); \
-	MCP_LOAD_OPT(paramsVariable__, keyproxiedMapEntryName__);           \
-	keyproxiedMapEntryName__ = mrpt::DEG2RAD(keyproxiedMapEntryName__)
+#define MCP_LOAD_OPT_DEG(Yaml__, Var__)                                        \
+	Var__ = mrpt::RAD2DEG(Var__);                                              \
+	MCP_LOAD_OPT(Yaml__, Var__);                                               \
+	Var__ = mrpt::DEG2RAD(Var__)
 
 /** Macro to store a variable into a mrpt::containers::yaml (initials MCP)
  * dictionary, using as "key" the name of the variable.
@@ -792,13 +926,18 @@ std::ostream& operator<<(std::ostream& o, const yaml& p);
  * // loaded in memory:
  * MCP_SAVE_DEG(p,K);
  * \endcode
+ *
+ * Since MRPT 2.3.2, this also works for enums, converting to textual names of
+ * values. Note that this requires enums to implement mrpt::typemeta::TEnumType.
  */
-#define MCP_SAVE(paramsVariable__, keyproxiedMapEntryName__) \
-	paramsVariable__[#keyproxiedMapEntryName__] = keyproxiedMapEntryName__;
+#define MCP_SAVE(Yaml__, Var__)                                                \
+	if constexpr (std::is_enum_v<decltype(Var__)>)                             \
+		Yaml__[#Var__] = mrpt::typemeta::TEnumType<                            \
+			std::remove_cv_t<decltype(Var__)>>::value2name(Var__);             \
+	else                                                                       \
+		Yaml__[#Var__] = Var__;
 
-#define MCP_SAVE_DEG(paramsVariable__, keyproxiedMapEntryName__) \
-	paramsVariable__[#keyproxiedMapEntryName__] =                \
-		mrpt::RAD2DEG(keyproxiedMapEntryName__);
+#define MCP_SAVE_DEG(Yaml__, Var__) Yaml__[#Var__] = mrpt::RAD2DEG(Var__);
 
 }  // namespace mrpt::containers
 
@@ -837,12 +976,17 @@ const T& yaml::asRef() const
 	const auto& storedType = s.type();
 
 	if (storedType != expectedType)
+	{
+		std::stringstream ss;
+		yaml::internalPrintAsYAML(s, ss, {}, {});
+
 		THROW_EXCEPTION_FMT(
-			"Trying to read parameter `%s` of type `%s` as if it was "
-			"`%s` and no obvious conversion found.",
-			proxiedMapEntryName_.c_str(),
+			"Trying to read parameter `%s` (value='%s') of type `%s` as if it "
+			"was `%s` and no obvious conversion found .",
+			proxiedMapEntryName_.c_str(), ss.str().c_str(),
 			mrpt::demangle(storedType.name()).c_str(),
 			mrpt::demangle(expectedType.name()).c_str());
+	}
 
 	return *std::any_cast<T>(&s);
 }
@@ -932,6 +1076,68 @@ inline void yaml::loadFromYAMLCPP(const YAML_NODE& n)
 	*this = yaml::FromYAMLCPP(n);
 }
 
+template <typename MATRIX>
+inline yaml yaml::FromMatrix(const MATRIX& m)
+{
+	yaml r = mrpt::containers::yaml::Map();
+	r["rows"] = static_cast<int64_t>(m.rows());
+	r["cols"] = static_cast<int64_t>(m.cols());
+	r["data"] = mrpt::containers::yaml::Sequence();
+	auto data = r["data"];
+	data.node().printInShortFormat = true;
+	for (int iRow = 0; iRow < m.rows(); iRow++)
+		for (int iCol = 0; iCol < m.cols(); iCol++)
+			data.push_back(m(iRow, iCol));
+	return r;
+}
+
+template <typename MATRIX>
+inline void yaml::toMatrix(MATRIX& m) const
+{
+	ASSERT_(isMap());
+	ASSERT_(has("rows") && has("cols") && has("data"));
+	const int nRows = (*this)["rows"].as<int>();
+	const int nCols = (*this)["cols"].as<int>();
+	ASSERT_((nRows > 0 && nCols > 0) || (nRows == 0 && nCols == 0));
+
+	const auto& data = (*this)["data"];
+	ASSERT_(data.isSequence());
+	ASSERT_EQUAL_(static_cast<int>(data.size()), nRows * nCols);
+
+	using entry_t = std::decay_t<decltype(m(0, 0))>;
+
+	if (m.cols() <= 0 || m.rows() <= 0)
+	{
+		try
+		{
+			m.resize(nRows, nCols);
+		}
+		catch (const std::exception&)
+		{
+		}
+	}
+	ASSERT_EQUAL_(m.cols(), nCols);
+	ASSERT_EQUAL_(m.rows(), nRows);
+
+	for (int r = 0, idx = 0; r < nRows; r++)
+		for (int c = 0; c < nCols; c++, idx++)
+			m(r, c) = data.operator()(idx).as<entry_t>();
+}
+
+template <typename Scalar>
+inline std::vector<Scalar> yaml::toStdVector() const
+{
+	ASSERT_(isSequence());
+	const auto& seq = asSequence();
+
+	std::vector<Scalar> ret;
+	ret.reserve(seq.size());
+
+	for (const auto& n : seq)
+		ret.push_back(n.as<Scalar>());
+	return ret;
+}
+
 /** Sort operator required for std::map with node_t as key */
 inline bool operator<(const yaml::node_t& lhs, const yaml::node_t& rhs)
 {
@@ -953,7 +1159,27 @@ T implAnyAsGetter(const mrpt::containers::yaml::scalar_t& s)
 	// 1) Exact match?
 	if (storedType == expectedType) return std::any_cast<const T&>(s);
 
-	// 2) Recognize double/float:
+	// 2) bool:
+	if constexpr (std::is_same_v<T, bool>)
+	{
+		if (storedType == typeid(std::string))
+		{
+			const auto str = implAnyAsGetter<std::string>(s);
+			std::optional<int> intVal;
+
+			char* retStr = nullptr;
+			const long long ret =
+				std::strtoll(str.c_str(), &retStr, 0 /*auto base*/);
+			if (retStr != 0 && retStr != str.c_str()) intVal = ret;
+
+			return str == "y" || str == "Y" || str == "yes" || str == "Yes" ||
+				str == "YES" || str == "true" || str == "True" ||
+				str == "TRUE" || str == "on" || str == "ON" || str == "On" ||
+				(intVal.has_value() && intVal.value() != 0);
+		}
+	}
+
+	// 3) Recognize double/float:
 	if constexpr (std::is_same_v<T, double> || std::is_same_v<T, float>)
 	{
 		if (storedType == typeid(double))
@@ -968,25 +1194,39 @@ T implAnyAsGetter(const mrpt::containers::yaml::scalar_t& s)
 		if (!ss.fail()) return ret;
 	}
 
-	// 3) Integers. Recognize hex or octal prefixes with stol()
+	// 4) Integers. Recognize hex or octal prefixes with strtol()
 	if constexpr (std::is_convertible_v<int, T>)
 	{
 		std::stringstream ss;
 		yaml::internalPrintAsYAML(s, ss, {}, {});
 		const std::string str = ss.str();
-		try
+
+		char* retStr = nullptr;
+		const long long ret =
+			std::strtoll(str.c_str(), &retStr, 0 /*auto base*/);
+		if (retStr != 0 && retStr != str.c_str())
 		{
-			std::size_t processed = 0;
-			T ret = static_cast<T>(
-				std::stol(str, &processed, 0 /*auto detect base*/));
-			if (processed > 0) return ret;
-		}
-		catch (...)
-		{ /*Invalid number*/
+			const auto minVal =
+				static_cast<long long>(std::numeric_limits<T>::min());
+			auto maxVal = static_cast<long long>(std::numeric_limits<T>::max());
+			// Handle the case of unsigned long long:
+			if (maxVal < 0) maxVal = std::numeric_limits<long long>::max();
+
+			if ((ret == 0 && errno == ERANGE) || ret < minVal || ret > maxVal)
+			{
+				std::stringstream sError;
+				sError << "yaml: Out of range integer: '" << str
+					   << "' (Valid range [" << minVal << "," << maxVal
+					   << "], parsed=" << ret;
+				if (errno == ERANGE) sError << " errno=ERANGE";
+				sError << "')";
+				THROW_EXCEPTION(sError.str());
+			}
+			return static_cast<T>(ret);
 		}
 	}
 
-	// 4) Strings:
+	// 5) Strings:
 	if constexpr (std::is_convertible_v<std::string, T>)
 	{
 		if (expectedType == typeid(std::string))
@@ -996,23 +1236,14 @@ T implAnyAsGetter(const mrpt::containers::yaml::scalar_t& s)
 			return ss.str();
 		}
 	}
-	// 5) bool:
-	if constexpr (std::is_same_v<T, bool>)
-	{
-		if (storedType == typeid(std::string))
-		{
-			const auto str = implAnyAsGetter<std::string>(s);
-			return str == "y" || str == "Y" || str == "yes" || str == "Yes" ||
-				   str == "YES" || str == "true" || str == "True" ||
-				   str == "TRUE" || str == "on" || str == "ON" || str == "On";
-		}
-	}
 
 	// No known way to convert it:
+	std::stringstream ss;
+	yaml::internalPrintAsYAML(s, ss, {}, {});
 	THROW_EXCEPTION_FMT(
-		"Trying to access scalar of type `%s` as if it was "
-		"`%s` and no obvious conversion found.",
-		mrpt::demangle(storedType.name()).c_str(),
+		"Trying to access scalar (value='%s') of type `%s` as if it was `%s` "
+		"and no obvious conversion found .",
+		ss.str().c_str(), mrpt::demangle(storedType.name()).c_str(),
 		mrpt::demangle(expectedType.name()).c_str());
 }
 
